% EM
% 
% Thomas Schn
% 22/4 - 2009

clear all;
close all;

randn('state',100);
rand('state',100);
addpath('H:\Matlab\unit');    % Add path to Adrian and Brett toolbox
%addpath('C:\Local Homes\schon\Research\unit\unit');    % Add path to UNIT toolbox

opt.allP  = 1;      % Store all covariance matrices
opt.miter = 100;    % Maximum number of updates of estimate from initial guess
opt.LLdec = 1e-6;   % Minimum decrease of the log-likelihood before the algorithm is terminated

theta0 = 0.1;       % Initial guess for the a-parameter
%========================================================
%===   Simulate a one dimensional state-space model   ===
%========================================================
MC   = 1000;           % Number of Monte Carlo simulations to perform
Ntot = [100 200 500 1000 2000 5000 10000];          % Number of samples

aHat = [];
disp(['Number of Monte Carlo simulations: ',num2str(MC)]);
for N = Ntot         % Loop over the number of samples
  disp(['Number of samples: ',num2str(N)]);
  clear a;  clear m;
  for i = 1:MC
    m.ss.A = 0.9;     m.ss.B = 0;
    m.ss.C = 0.5;     m.ss.D = 0;
    m.ss.Q = 0.1;     m.ss.S = 0;
    m.ss.R = 0.1;

    m.ss.X1 = 0;      m.ss.P1 = 0;
    x = zeros(1,N+1); y = zeros(1,N);
    x(1) = m.ss.X1;                 % Initial state value (fully known)
    v = sqrt(m.ss.Q)*randn(1,N);    % Process noise sequence
    e = sqrt(m.ss.R)*randn(1,N);    % Measurement noise sequence
    for t=1:N
      x(t+1) = m.ss.A*x(t) + v(t);
      y(t)   = m.ss.C*x(t) + e(t);
    end
    z.y = y;

    mEst = m;
    mEst.ss.A = theta0;     % Initial parameter guess
    %========================
    %===   EM algorithm   ===
    %========================
    for k = 1:opt.miter
      % E step
      g     = ks(z,mEst,opt);       % Smoother
      LL(k) = -(0.5)*g.LL;          % Store the log-likelihood
      phi   = 0;   psi = 0;   sigma = 0;
      for t = 1:N
        phi   = phi + g.ss.xs(t+1)*g.ss.xs(t+1) + g.ss.Ps(1,1,t+1)^2;
        sigma = sigma + g.ss.xs(t).^2 + g.ss.Ps(1,1,t)^2;
        psi   = psi + g.ss.xs(t)*g.ss.xs(t+1) + g.ss.Ms(1,1,t);
      end;
      % M step
      a{i}(k) = psi/sigma;
      mEst.ss.A = a{i}(k);
      if k>1          % Check termination condition
        if LL(k)-LL(k-1) < opt.LLdec
          break;
        end;
      end;
    end;
    a{i} = [theta0 a{i}];    % Store all the iterates
  end;
  % Compute the MC mean estimate
  for i = 1:MC
    aEst(i) = a{i}(end);     % Extract the final a estimate
  end;
  aHat = [aHat mean(aEst)];                 % Compute the MC mean
end;
%==============================
%===   Analyze the result   ===
%==============================
